@@ -5,8 +5,7 @@ test_that("sim_beta returns an object with certain characteristics",{
   results <- sim_beta(simResults, alpha)
 
   testthat::expect_s3_class(results, "ecocbo_beta")
-  testthat::expect_equal(nrow(results$Power),
-                         (max(simResults$Results[,3])-1) * (max(simResults$Results[,4])-1))
+  testthat::expect_equal(nrow(results$Power), 4)
   testthat::expect_error(sim_beta(simResults, alpha = 2), "smaller")
 })
 
